<?php
/**
 * The template for displaying product content within loops
 *
 * This template can be overridden by copying it to yourtheme/woocommerce/content-product.php.
 *
 * HOWEVER, on occasion WooCommerce will need to update template files and you
 * (the theme developer) will need to copy the new files to your theme to
 * maintain compatibility. We try to do this as little as possible, but it does
 * happen. When this occurs the version of the template file will be bumped and
 * the readme will list any important changes.
 *
 * @see     https://docs.woocommerce.com/document/template-structure/
 * @package WooCommerce\Templates
 * @version 3.6.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	die();
}

global $product, $this_item_count;

// Ensure visibility.
if ( empty( $product ) || ! $product->is_visible() ) {
	return;
}
?>

<?php
	$column_size = wc_get_loop_prop( 'columns' );
	$classes     = (int) ( 12 / $column_size );

	$this_item_count++;

	$classes = 'col-md-' . $classes . ' col-sm-6';
?>

<li class="<?php echo esc_attr( $classes ); ?>">

	<div <?php wc_product_class(); ?> >

		<div class="themeoo-product-item">
			<div class="product-thumb">
				<?php
				themeoo_the_post_thumbnail(
					'shop-product-image',
					[
						'wrapper'   => false,
						'auto_size' => true,
					]
				);
				?>
				<div class="product-badge-wrap">
					<?php themeoo_get_product_discount(); ?>
					<div class="divider"></div>
					<?php themeoo_get_product_badge( get_the_ID(), array( 'hot', 'sale' ) ); // @phpstan-ignore-line ?>
				</div>
				<div class="product-actions">
					<?php
					if ( class_exists( 'YITH_WCQV' ) ) {
						echo do_shortcode( sprintf( '[yith_quick_view label="<i class=\'ti-zoom-in\'></i><span class=\'themeoo-tooltip\'>%s</span>"]', __( 'Quick View', 'ayyash' ) ) );
					}
					?>
					<?php
					if ( class_exists( 'YITH_Woocompare' ) ) {
						echo do_shortcode( sprintf( '[yith_compare_button]<i class="ti-control-shuffle" aria-hidden="true"></i><span class="themeoo-tooltip">%s</span>[/yith_compare_button]', __( 'Compare', 'ayyash' ) ) );
					}
					?>
					<?php woocommerce_template_loop_add_to_cart(); ?>
				</div>
			</div>

			<div class="product-content">
				<?php if ( ! empty( $product->get_average_rating() ) ) : ?>
				<div class="rating">
					<?php echo wc_get_rating_html( $product->get_average_rating() ); //phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped ?>
				</div>
				<?php endif; ?>

				<?php if ( ! empty( get_the_title() ) ) : ?>
				<h3 class="themeoo-product-title"><a href="<?php the_permalink(); ?>"><?php the_title(); ?></a></h3>
				<?php endif; ?>
				<div class="product-price-wrap">
					<?php wc_get_template( 'loop/price.php' ); ?>
				</div>
			</div>
		</div>

	</div>
</li>
<?php
if ( ( $this_item_count % $column_size ) == 0 ) {
	echo '</ul>';
	echo '<ul class="' . esc_attr( 'row products themeoo-products columns-' . $column_size ) . '">';
}
